{% extends "allauth/layouts/entrance.html" %}
{% load allauth %}
{% block content %}
    {% element h1 %}
        Example Project
    {% endelement %}
    <p>Welcome to the django-allauth example project.</p>
{% endblock content %}
